#ifndef SCIP2_H
#define SCIP2_H
/*!
  @file
  @brief SCIP2 API
  @author aokaze (HARA Yoshitaka) <bluewind@roboken.esys.tsukuba.ac.jp>
  @author YoS (YOSHIDA Tomoaki) <yoshida@furo.org>
  @par    Publisher:
          HOKUYO AUTOMATIC CO.,LTD.
  @since  2006
  @date   2007/03/30
*/

#include <sys/time.h>
#include "scipBase.h"

//! SCIP2.0 モードに移行
/*!
  URG シリーズ特有のコマンドを使用して SCIP2.0 モードに移行する。
  @param aPort [in] センサとの通信ハンドル
  @retval 0 成功
  @retval -1 コマンドの送受信に失敗
  @retval -2 不良伝文応答（未定義コマンド）
  @retval other 失敗（応答ステータス）
*/
int switchToScip2(tPort* aPort);

//! センサのバージョン情報表示[VVコマンド]
/*!
  @param aPort [in] センサとの通信ハンドル
  @retval 0 成功
  @retval -1 コマンドの送受信に失敗
  @retval -2 不良伝文応答（未定義コマンド）
  @retval other 失敗（応答ステータス）
*/
int scip2ShowVersion(tPort* aPort);

//! センサのパラメータ情報表示[PPコマンド]
/*!
  @param aPort [in] センサとの通信ハンドル
  @retval 0 成功
  @retval -1 コマンドの送受信に失敗
  @retval -2 不良伝文応答（未定義コマンド）
  @retval other 失敗（応答ステータス）
*/
int scip2ShowParameter(tPort* aPort);

//! センサのステータス情報表示[IIコマンド]
/*!
  @param aPort [in] センサとの通信ハンドル
  @retval 0 成功
  @retval -1 コマンドの送受信に失敗
  @retval -2 不良伝文応答（未定義コマンド）
  @retval other 失敗（応答ステータス）
*/
int scip2ShowStatus(tPort* aPort);

//! センサおよびホストの時刻表示[TMコマンド]
/*!
  @param aPort [in] センサとの通信ハンドル
  @retval 0 成功
  @retval -1 コマンドの送受信に失敗
  @retval -2 不良伝文応答（未定義コマンド）
  @retval other 失敗（応答ステータス）
*/
int scip2ShowTime(tPort* aPort);

//! 通信速度を変更[SSコマンド]
/*!
  @param aPort [in] センサとの通信ハンドル
  @param aSpeed [in] 設定する通信速度 (bps)
  @retval 0 成功
  @retval -1 コマンドの送受信に失敗
  @retval -2 不良伝文応答（未定義コマンド）
  @retval -3 通信速度設定に失敗
  @retval other 失敗（応答ステータス）
*/
int scip2SetComSpeed(tPort* aPort, int aSpeed);

//! 走査回転速度変更[CRコマンド]
/*!
  @param aPort [in] センサとの通信ハンドル
  @param aRate [in] 600 rpm からの減速率（％）（99の場合は初期速度） (0-10,99)
  @retval 0 成功
  @retval -1 コマンドの送受信に失敗
  @retval -2 不良伝文応答（未定義コマンド）
  @retval -3 指定減速率不良（0-10,99以外）
  @retval other 失敗（応答ステータス）
*/
int scip2SetScanSpeed(tPort* aPort, int aRate);

//! 新規測定してスキャンデータを取得[MD/MSコマンド]
/*!
  @param aPort [in] センサとの通信ハンドル
  @param aStartStep [in] スキャンの開始方向 (0-768)
  @param aEndStep [in] スキャンの終了方向 (0-768)
  @param aStepCluster [in] データ取得時にまとめる方向数 (0-99)
  @param aScanInterval [in] データ取得時に間引くスキャン数 (0-9)
  @param aScanNum [in] データ取得を行うスキャン数 (0-99)
  @param aEncType [in] データのエンコーディングタイプ (ENC_3BYTE, ENC_2BYTE)
  @param aStepNum [out] スキャンデータのステップ数 (1-769)
  @return 取得したスキャン数分の2次元距離データ配列の先頭アドレス
  malloc したメモリへのポインタなので、使用後に free する必要がある。
  スキャン取得に失敗したときは NULL を返す。
*/
int** scip2MeasureScan(tPort* aPort, int aStartStep, int aEndStep, int aStepCluster, int aScanInterval, int aScanNum, EncType_t aEncType, int *aStepNum);

//! 常時測定してスキャンデータを保持（レーザ点灯）[BMコマンド]
/*!
  @param aPort [in] センサとの通信ハンドル
  @retval 0 成功
  @retval -1 コマンドの送受信に失敗
  @retval -2 不良伝文応答（未定義コマンド）
  @retval other 失敗（応答ステータス）
*/
int scip2StartStoringScan(tPort* aPort);

//! 測定（MD/MS,BMコマンド）を停止（レーザ消灯）[QTコマンド]
/*!
  現状ではMD/MSコマンドに対応する関数（scip2MeasureScan）は Blocking 実装のため、
  本関数（scip2StopStoringScan）はBMコマンド（scip2StartStoringScan）にのみ対応。
  @param aPort [in] センサとの通信ハンドル
  @retval 0 成功
  @retval -1 コマンドの送受信に失敗
  @retval -2 不良伝文応答（未定義コマンド）
  @retval other 失敗（応答ステータス）
*/
int scip2StopStoringScan(tPort* aPort);

//! 測定済みスキャンデータを取得[GD/GSコマンド]
/*!
  @param aPort [in] センサとの通信ハンドル
  @param aStartStep [in] スキャンの開始方向 (0-768)
  @param aEndStep [in] スキャンの終了方向 (0-768)
  @param aStepCluster [in] データ取得時にまとめる方向数 (0-99)
  @param aEncType [in] データのエンコーディングタイプ (ENC_3BYTE, ENC_2BYTE)
  @param aStepNum [out] スキャンデータのステップ数 (1-769)
  @return 取得したスキャンの距離データ配列の先頭アドレス
  malloc したメモリへのポインタなので、使用後に free する必要がある。
  スキャン取得に失敗したときは NULL を返す。
*/
int* scip2GetScan(tPort* aPort, int aStartStep, int aEndStep, int aStepCluster, EncType_t aEncType, int *aStepNum);

//! malloc したスキャンの距離データ配列を free
/*!
  @param aScan [in] aScan 破棄するスキャンの距離データ配列の先頭アドレス
  @param aScanNum [in] 距離データ配列の数（スキャン数）(0-99)
  @retval EXIT_SUCCESS 成功
  @retval EXIT_FAILURE 失敗
 */
int scip2FreeScan(int **aScan, int aScanNum);

//! センサリセット（全パラメータ初期化）[RSコマンド]
/*!
  @param aPort [in] センサとの通信ハンドル
  @retval 0 成功
  @retval -1 コマンドの送受信に失敗
  @retval -2 不良伝文応答（未定義コマンド）
  @retval -3 通信速度設定に失敗
  @retval other 失敗（応答ステータス）
*/
int scip2SensorReset(tPort* aPort);

#endif  // SCIP2_H
