#include <stdio.h>
#include "scipBase.h"

speed_t bitrate(int aBR)
{
  switch(aBR)
    {
    case 9600:
      return B9600;

    case 19200:
      return B19200;

    case 38400:
      return B38400;

    case 57600:
      return B57600;

    case 115200:
      return B115200;

    case 230400:
      return B230400;

    case 460800:
      return B460800;

    case 500000:
      return B500000;

    default:  // invalid bitrate
      return B0;
    }
}

speed_t getBitrate(int aFD)
{
  struct termios tio;

  tcgetattr(aFD,&tio);
  return cfgetospeed(&tio);
}

void skipReceiveBuf(FILE *aDev)
{
  char line[LINE_SIZE];

  tcflush(fileno(aDev),TCIOFLUSH);
  while(fgets(line,sizeof(line),aDev)!=NULL)
    if(strcmp(line,LF_STRING)!=0)
      fprintf(stdout,"skipping receive buffer : %s\n",line);
}

void flushSensorBuf(FILE* aDev)
{
  fwrite("\n",sizeof(char),1,aDev);
  usleep(100000);  // 100 ms （URG の応答時間）
  skipReceiveBuf(aDev);
}

int setupPort(int aFD, int aSpeed)
{
  speed_t speed;
  struct termios tio;

  speed=bitrate(aSpeed);
  if(speed==B0){
    fprintf(stderr,"INVALID BITRATE\n");
    return EXIT_FAILURE;  // invalid bitrate
  }

  tcgetattr(aFD,&tio);

  //   cfsetspeed(&tio,speed);  // bitrate
  cfsetospeed(&tio,speed);  // bitrate
  cfsetispeed(&tio,speed);  // bitrate

  cfmakeraw(&tio);
  tio.c_cflag&= ~CRTSCTS;  // no CTS/RTS  ハードウェアフロー制御なし
#if 0
  tio.c_iflag &= ~( BRKINT | ICRNL | ISTRIP );
  tio.c_iflag &= ~IXON;     // no XON/XOFF
  tio.c_cflag &= ~PARENB;   // no parity
  tio.c_cflag &= ~CSTOPB;   // stop bit = 1bit
  tio.c_cflag = (tio.c_cflag & ~CSIZE) | CS8;  // data bits = 8bit
  tio.c_lflag &= ~( ISIG | ICANON | ECHO );    // Other
#endif

  // 非カノニカルモードオプション
  tio.c_cc[VTIME] = 5;  // 500 ms to timeout
  tio.c_cc[VMIN] = 0;   // 最小文字数

  // Commit
  if(tcsetattr(aFD,TCSADRAIN,&tio)==0)
    return EXIT_SUCCESS;
  else
    return EXIT_FAILURE;
}

tPort* openPort(char *aDevName, int aSpeed)
{
  tPort* port;
  FILE* dev;

  dev=fopen(aDevName,"w+");
  if(dev==NULL)
    return NULL;  // invalid device file

  // setup parameters
  if(setupPort(fileno(dev),aSpeed)==EXIT_SUCCESS){
#if 1
    flushSensorBuf(dev);		// ok
#endif
    port=(tPort*)calloc(1,sizeof(tPort));
    port->dev=dev;
    return port;
  } else {  // fail
    perror("Could not set terminal parameters ");
    fclose(dev);
    return NULL;
  }
}

int closePort(tPort *aPort)
{
  if(!aPort)
    return EXIT_FAILURE;
  flushSensorBuf(aPort->dev);
  fclose(aPort->dev);
  free(aPort);
  return EXIT_SUCCESS;
}

int scipProtCheck(tPort* aPort, int aProtocol)
{
  if(aPort->protocol==aProtocol || aPort->protocol==0)
    return 1;
  else
    return 0;
}

int scipReceiveResEnd(tPort* aPort)
{
  char line[LINE_SIZE];

  if(fgets(line,sizeof(line),aPort->dev)==NULL)
    return -1;
  if(strcmp(line,LF_STRING)!=0){  // invalid
    fprintf(stderr,"This response is not the end code\n");
    skipReceiveBuf(aPort->dev);
    return -2;
  }
  return 0;
}

int scipDecode(char *aData, int aEncLen)
{
  int ret=0;
  while(aEncLen--){
    if(*aData<0x30 || *aData>0x6f){
      fprintf(stderr,"Could not decode : %s\n",aData);
      return -1;
    }
    ret= (ret<<6) + (*(aData++) - 0x30);
  }
  return ret;
}

int scipSendCmd(tPort* aPort, char *aCmd, int aProtocol)
{
  static const char SCIP1_NORMAL_STATUS[]={'0',LF,'\0'};
  static const char SCIP2_NORMAL_STATUS[]={'0','0','P',LF,'\0'};
  char line[LINE_SIZE];
  int i;

  if(!scipProtCheck(aPort,aProtocol)){
    fprintf(stderr,"INVALID VERSION SCIP COMMAND :\tThe sensor is not work in SCIP%d mode\n",aProtocol);
    return -2;
  }

  i=fwrite(aCmd,sizeof(char),strlen(aCmd),aPort->dev);
  if(i==0){
    perror("Could not send command ");
    return -1;
  }

  // echo-back
  if(fgets(line,sizeof(line),aPort->dev)==NULL){
    fprintf(stderr,"NO ECHO BACK\n");
    return -1;
  }
  if(strcmp(line,aCmd)!=0){
    // invalid
    fprintf(stderr,"INVALID ECHO BACK :\tExp = %s\tRec = %s\n",aCmd,line);
    skipReceiveBuf(aPort->dev);
    return -1;
  }

  // status
  if(fgets(line,sizeof(line),aPort->dev)==NULL){
    fprintf(stderr,"NO STATUS\n");
    return -1;
  }
// #ifdef DEBUG
#if 1
  fprintf(stdout,"status = %s",line);
#endif
  // バージョンチェックも必要か？しかし、scipTryComm() の時点ではバージョンは不明。
  if(!(strcmp(line,SCIP1_NORMAL_STATUS)==0) &&
     !(strcmp(line,SCIP2_NORMAL_STATUS)==0)){
    //invalid
    fprintf(stderr,"STATUS CHECK FAILED : %s",line);
    skipReceiveBuf(aPort->dev);
    i=atoi(line);	// lineという文字列を整数に変換
    if(i==0)
      i=-2;
    return i;
  }
#if 0
  aPort->protocol=aProtocol;
#endif
  return 0;
}
