#ifndef SCIP_BASE_H
#define SCIP_BASE_H
/*!
  @file
  @brief SCIP ベース API
  @author aokaze (HARA Yoshitaka) <bluewind@roboken.esys.tsukuba.ac.jp>
  @author YoS (YOSHIDA Tomoaki) <yoshida@furo.org>
  @par    Publisher:
          HOKUYO AUTOMATIC CO.,LTD.
  @since  2005
  @date   2007/03/30
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/termios.h>
#include <unistd.h>
#include <math.h>

// hardcode parameters for URG
// TODO: SCIP2 コマンド（PPコマンド）によるパラメータ取得
#define FRONT_STEP 384
#define ANG_RESOLUTION 1024

#define LINE_SIZE 80
#define CMD_SIZE 33
#define LF 0x0A
static const char LF_STRING[]={LF,'\0'};

//! センサの動作状態
typedef enum
{
  STATE_UNKNOWN=0,      //!< 状態不明
  STATE_IDLE,           //!< アイドリング（レーザOFF）
  STATE_MEASURE_SEND,   //!< 常時測定・データ送信（レーザON、垂れ流しモード）
  STATE_MEASURE_STORE,  //!< 常時測定・データ保持（レーザON、クラシックモード）
  STATE_TIME_ADJUST     //!< 時計合わせモード
}State_t;

//! データのエンコーディングタイプ
typedef enum
{
  ENC_2BYTE=2,  //!< 2キャラエンコーディング
  ENC_3BYTE,    //!< 3キャラエンコーディング
  ENC_4BYTE     //!< 4キャラエンコーディング（タイムスタンプ用）
}EncType_t;

//! センサとの通信ハンドル
typedef struct
{
  FILE *dev;     //!< センサが接続されているポートのファイルハンドル
  int protocol;  //!< プロトコルメジャーバージョン（0 の場合は UNKNOWN）
  State_t state; //!< センサの動作状態
}tPort;


// シリアルポート関連関数群

//! 通信速度設定値を決定
/*!
  @param aBR [in] 指定する通信速度 (9600-500000bps)
  750000bps には対応していない
  @return cfset[i|o]speed 関数に渡す値
  失敗したときは B0 を返す
*/
speed_t bitrate(int aBR);

//! シリアルポートの現在の通信速度設定値を取得
/*!
  @param aFD [in] シリアルポートを指すファイルディスクリプタ
  @return 現在の通信速度設定値
*/
speed_t getBitrate(int aFD);

//! 受信バッファに残っているデータを読み飛ばす
/*!
  @param aDev [in] センサが接続されているポートのファイルハンドル
*/
void skipReceiveBuf(FILE *aDev);

//! センサ側のバッファをクリア
/*!
  センサ側のバッファをクリアするために LF を送信する。シリアル通信
  （シリアルポート、またはUSB-シリアル変換器）の信頼性が高い場合、
  および USB の場合はおそらく使用する必要はない。この関数を使用しない
  場合、応答は早くなるが通信の失敗率が上がってしまう。
  @param aDev [in] センサが接続されているポートのファイルハンドル
*/
void flushSensorBuf(FILE* aDev);

//! シリアルポートの通信パラメータを設定
/*!
  @param aFD [in] シリアルポートを指すファイルディスクリプタ
  @param aSpeed [in] 設定する通信速度 (bps)
  @retval EXIT_SUCCESS 成功
  @retval EXIT_FAILURE 失敗
*/
int setupPort(int aFD, int aSpeed);

//! 指定されたシリアルポートを開いてセットアップ
/*!
  @param aDevName [in] シリアルポートのデバイスファイル名
  @param aSpeed [in] 通信速度 (bps)
  @return センサとの通信ハンドル
  失敗したときは NULL を返す
*/
tPort* openPort(char *aDevName, int aSpeed);

//! 指定された通信ハンドルを閉じる
/*!
  @param aPort [in] センサとの通信ハンドル
  @retval EXIT_SUCCESS 成功
  @retval EXIT_FAILURE 失敗
*/
int closePort(tPort *aPort);


// SCIP コマンド送受信ベース関数

//! センサとの通信ハンドルのプロトコルメジャーバージョンをチェック
/*!
  @param aPort [in] センサとの通信ハンドル
  @param aProtocol [in] SCIP コマンドのプロトコルメジャーバージョン
  @retval 1 プロトコルのメジャーバージョンが一致（または UNKNOWN）
  @retval 0 プロトコルのメジャーバージョンが不一致
*/
int scipProtCheck(tPort* aPort, int aProtocol);

//! SCIP コマンド（リクエスト）を送信して応答ステータスまでの応答（レスポンス）を受信
/*!
  aDev に接続されたセンサに aCmd で与えられる SCIP コマンドを送信し、
  その応答のうちステータスまでを受信する。ステータス以降のデータは
  コマンド毎に長さ（行数）が異なるため受信しない。よって終端コード
  （LF 2個）の残りの LF 1個、または「データ+終端コード」を別途受信する
  必要あり。
  @param aPort [in] センサとの通信ハンドル
  @param aCmd [in] 送信する SCIP コマンド文字列
  @param aProtocol [in] 送信する SCIP コマンドのプロトコルメジャーバージョン
  @retval 0 成功
  @retval -1 コマンドの送受信に失敗
  @retval -2 不良伝文応答（未定義コマンド）
  @retval other 失敗（応答ステータス）
*/
int scipSendCmd(tPort* aPort, char *aCmd, int aProtocol);

//! センサからの応答（レスポンス）の最後の LF を受信
/*!
  @param aPort [in] センサとの通信ハンドル
  @retval 0 成功
  @retval -1 応答（レスポンス）の受信に失敗
  @retval -2 応答（レスポンス）が終端コードの LF でない
*/
int scipReceiveResEnd(tPort* aPort);

//! エンコード済みデータをデコード
/*!
  @param aData [in] エンコード済みデータ
  @param aEncLen [in] エンコーディングタイプ（キャラクターエンコード法のキャラクター数）
  @return デコードしたデータ
*/
int scipDecode(char *aData, int aEncLen);

#endif  // SCIP_BASE_H
